#! /usr/bin/env python

import os
import shutil

from configParser import parser
import nuoParser as nps
import siteData as sd

def main():
    if(os.path.exists("build")):
        shutil.rmtree("build")
    os.mkdir("build")
    print("Starting to parse...", end = "\n\n")
    parser.parse("./config.yml")
    sd.siteData()
    nps.parse()



import siteData as sd
from .. import patterns as p
from .utils import chainedPropertyAccess

#Function to parse variables in a string expression
def exp(expression):

    #List to store parsed terms as per input string (expression)
    modExpression = []

    #Variable to define offest for next term in resultant string
    z = 0

    for elem in p._var.finditer(expression):
        x, y = elem.span()
        tempVar = expression[x + 2:y - 2].strip().split(".")

        modExpression.append(expression[z:x])

        #Check for globals
        if(len(tempVar) == 1 and tempVar[0] in sd.GLOBALS.keys()):
            modExpression.append(str(sd.GLOBALS[tempVar[0]]))

        #Check for objects in data files
        elif(len(tempVar) == 1 and tempVar[0] in sd.DATAOBJECT.keys()):
            modExpression.append(str(sd.DATAOBJECT[tempVar[0]]))

        #Check for objects in defined objeccts
        elif(len(tempVar) == 1 and tempVar[0] in sd.DEFINEDOBJECTS.keys()):
            modExpression.append(str(sd.DEFINEDOBJECTS[tempVar[0]]))

        #Check for object properties
        elif(len(tempVar) > 1):
            root = tempVar[0]

            #Check for object properties in globals
            if(root in sd.GLOBALS.keys()):
                value = chainedPropertyAccess(sd.GLOBALS[root], tempVar[1:])
                modExpression.append(str(value))

            #Check for object properties in data files
            elif(root in sd.DATAOBJECT.keys()):
                value = chainedPropertyAccess(sd.DATAOBJECT[root], tempVar[1:])
                modExpression.append(str(value))

            #Check for object properties in earlier defined objects
            elif(root in sd.DEFINEDOBJECTS.keys()):
                value = chainedPropertyAccess(sd.DEFINEDOBJECTS[root], tempVar[1:])
                modExpression.append(str(value))

        #Store offset value for next term in resultant string
        z = y

    #Append trailing term in input expression
    modExpression.append(expression[z:])

    #Return parsed string
    return "".join(modExpression).strip()



import os
import re

from siteData import *
from . import patterns as p
from .action import setAction, takeAction, openFile, closeFile
from .handlers import var, define, rangeHandler, arithmetic as a

def start():
    files = os.listdir(NUODIR)
    parse(files = files)

def parse(files = [], prefix = NUODIR):
    pattern = re.compile("^.*\.nuo$")
    for file in files:
        file = os.path.join(prefix, file)
        if(os.path.isdir(file)):
            os.mkdir("build/" + file[len(NUODIR):])
            parse(os.listdir(file), file)
        if pattern.match(file) is not None:
            parseFile(file)
            pass
        else:
            pass

def parseFile(file):
    if os.path.exists(file):
        baseName = file[len(NUODIR):]
        outputFileName = "build/" + baseName.split(".")[0] + ".html"
        openFile(outputFileName)

        f = open(file, "r")
        for line in f.readlines():
            method = detectPattern(line)

            if method == "var":
                line = var.exp(line)
                takeAction(line)

            if method == "define":
                define.exp(line)

            if method == "range":
                setAction("range")
                rangeHandler.startRangeBlock(line)

            if method == "rangeEnd":
                setAction("rangeEnd")
                takeAction(line)

            if method == "arithmetic":
                line = a.eval(line)
                takeAction(line)

            if method == "raw":
                takeAction(line)

        closeFile()

    else:
        raise Exception("File '{}' does not exist".format(file))

def detectPattern(expression):
    if p._var.search(expression) is not None:
        return "var"

    if p._range.search(expression) is not None:
        return "range"

    if p._define.search(expression) is not None:
        return "define"

    if p._rangeEnd.search(expression) is not None:
        return "rangeEnd"

    if p._arithmetic.search(expression) is not None:
        return "arithmetic"

    return "raw"



import re

from .. import patterns as p
from .utils import getValue
from .. import action

RANGEOBJECT = {"isSet": False}
CURRENTRANGEOBJECT = RANGEOBJECT
RANGECOUNTER = 0

def _setRangeObject(identifier, iterValue):

    global RANGEOBJECT, CURRENTRANGEOBJECT, PREVRANGEOBJECT, RANGECOUNTER
    RANGECOUNTER += 1

    tempRangeObj = None
    if(CURRENTRANGEOBJECT["isSet"] is True):
        CURRENTRANGEOBJECT["body"].append({})
        tempRangeObj = CURRENTRANGEOBJECT
        CURRENTRANGEOBJECT = CURRENTRANGEOBJECT["body"][len(CURRENTRANGEOBJECT["body"]) - 1]

    CURRENTRANGEOBJECT["prev"] = tempRangeObj
    CURRENTRANGEOBJECT["iterValue"] = iterValue
    CURRENTRANGEOBJECT["id"] = identifier
    CURRENTRANGEOBJECT["body"] = []
    CURRENTRANGEOBJECT["isSet"] = True

def startRangeBlock(expression):

    for elem in p._range.finditer(expression):
        x, y = list(elem.span())
        tempExp = expression[x + 1:y - 1].strip().split()

        identifier, iterKey = list(tempExp[1].split(":"))

        _setRangeObject(identifier, iterKey)

def putLine(expression):
    global CURRENTRANGEOBJECT
    CURRENTRANGEOBJECT["body"].append(expression)

def endRangeBlock():
    global RANGEOBJECT, CURRENTRANGEOBJECT, RANGECOUNTER

    if(RANGECOUNTER is not 1):
        CURRENTRANGEOBJECT = CURRENTRANGEOBJECT["prev"]
        action.setAction("range")
        RANGECOUNTER -= 1
        return

    action.setAction("file")
    RANGETEMPDATA = {}
    RANGECOUNTER -= 1

    def parseRangeExp(id, line):

        if(re.compile("{[a-zA-Z0-9_\-\.]+}").search(line) is not None):
            modExpression = []
            z = 0

            for elem in re.compile("{[a-zA-Z0-9_\-\.]+}").finditer(line):

                x, y = list(elem.span())
                tempExp = line[x + 1:y - 1].strip().split(".")
                modExpression.append(line[z:x])

                _v = getValue(tempExp, True, RANGETEMPDATA)
                modExpression.append(str(_v))

                z = y

            modExpression.append(line[z:])
            return "".join(modExpression)
        else:
            return line

    def _parseRangeBody(id, rb):
        for i in rb:
            if(type(i) is dict):
                parseRange(i)
            else:
                i = i.strip()
                action.takeAction(parseRangeExp(id, i))

    def parseRange(rangeObj):

        if(type(rangeObj["iterValue"]) is str and rangeObj["iterValue"].isdigit()):
            rangeObj["iterValue"] = int(rangeObj["iterValue"])
        elif(type(rangeObj["iterValue"]) is str):
            iterKey = rangeObj["iterValue"].split(".")
            rangeObj["iterValue"] = getValue(iterKey, True, RANGETEMPDATA)

        t = type(rangeObj["iterValue"])
        Iter = _getIter(t, rangeObj["iterValue"])
        if(t is not dict):
            for i in Iter:
                RANGETEMPDATA[rangeObj["id"]] = i
                _parseRangeBody(rangeObj["id"], rangeObj["body"])
        else:
            RANGETEMPDATA[rangeObj["id"]] = {}
            for k, v in Iter.items():
                RANGETEMPDATA[rangeObj["id"]]["k"] = k
                RANGETEMPDATA[rangeObj["id"]]["v"] = v
                _parseRangeBody(rangeObj["id"], rangeObj["body"])

    parseRange(RANGEOBJECT)
    RANGEOBJECT = {"isSet": False}
    RANGECOUNTER = 0
    CURRENTRANGEOBJECT = RANGEOBJECT

def _getIter(t, val):

    if(t is int):
        return range(val)
    else:
        return val



import siteData as sd
from .. import patterns as p
from .utils import chainedPropertyAccess

#Function to define variables as parsed from nuo files
def exp(expression):

    #Iterate over matched regexp expressions
    for elem in p._define.finditer(expression):
        x, y = list(elem.span())

        #Variable that holds expression within curly bounds
        tempExp = expression[x + 2: y - 2].strip().split()

        #identifier of the value to be assigned, Referring to an already assigned object
        refObj = tempExp[2].split(".")
        root = refObj[0]

        #Check for the existence of object in glboal variables
        if(root in sd.GLOBALS.keys()):

            #Check if there is no property chaining
            if(len(refObj) == 1):
                val = sd.GLOBALS[root]
            else:
                val = chainedPropertyAccess(sd.GLOBALS[root], refObj[1:])

        #Check for the existence of object in in data files
        elif(root in sd.DATAOBJECT.keys()):

            #Check is there is no property chaining
            if(len(refObj) == 1):
                val = sd.DATAOBJECT[root]
            else:
                val = chainedPropertyAccess(sd.DATAOBJECT[root], refObj[1:])

        #Check for the existence of object in earlier defined objects
        elif(root in sd.DEFINEDOBJECTS.keys()):

            #Check is there is no property chaining
            if(len(refObj) == 1):
                val = sd.DEFINEDOBJECTS[root]
            else:
                val = chainedPropertyAccess(sd.DEFINEDOBJECTS[root], refObj[1:])

        #Set the value of key as per expression in nuo file
        sd.DEFINEDOBJECTS[tempExp[1]] = val



import os

__all__ = ["GLOBALS", "NUODIR", "DATAFILESOBJECT", "DATAOBJECT", "DEFINEDOBJECTS", "setNuoDir", "siteData"]

GLOBALS = {}

NUODIR = os.getcwd()

DATAFILESOBJECT = {}

DATAOBJECT = {}

DEFINEDOBJECTS = {}  #Objects defined during parser runtime with "define" directive

def setNuoDir(nuoDir):
    global NUODIR
    NUODIR = nuoDir

def siteData(verbose = False):
    if(verbose):
        print("Globals: {}".format(GLOBALS), end = "\n\n")
        print("NuoFiles Directory: {}".format(NUODIR), end = "\n\n")
        print("Data Files: {}".format(DATAFILESOBJECT), end = "\n\n")



from .. import patterns as p
from .utils import getPostfix, evalPostfix, getValue

def eval(expression):

	operators = ["+", "-", "/", "*"]
	modExpression = []
	z = 0
	for elem in p._arithmetic.finditer(expression):

		x, y = elem.span()
		tempExp = expression[x + 2:y - 2]
		modExpression.append(expression[z:x])

		pFix = getPostfix(tempExp)
		pFix = pFix.split(" ")
		for i in range(len(pFix)):

			ch = pFix[i]
			if(not ch.isdigit() and ch is not "" and ch not in operators):
				ch = ch.split(".")
				pFix[i] = str(getValue(ch))
		modExpression.append(str(evalPostfix(" ".join(pFix))))
		z = y
	modExpression.append(expression[z:])
	return "".join(modExpression)



import os

from .handlers import rangeHandler

_openedFile = None

ACTION = "file"

HTML_DIRECTORY = os.path.abspath("build")

"""Set to hold actions defined in nuoParser
Description of actions

"file": To write a line to build file"""
actionSet = {"file", "range", "rangeEnd"}

#Function ot set current action for nuoParser
def setAction(action):

    #Refer to ACTION variable outside function scope
    global ACTION

    #Check if action is present is actionSet
    if(action not in actionSet):
        raise Exception("Action '{}' is not defined".format(action))

    #Set ACTION to specified action
    ACTION = action

#Function to open output html file in "build/" folder by default
def openFile(file):

    #Refer to _openedFile variable outside function scope
    global _openedFile
    _openedFile = open(file, "+a")

#Function to close output html file referd by _openedFile global variable
def closeFile():

    #Refer to _openedFile varaible outside function scope
    global _openedFile
    _openedFile.close()  #Close file
    _openedFile = None  #Switch to None object

#Function to execute current action for the nuoParser
def takeAction(line):

    #Check if current action is file
    if(ACTION == "file"):

        #Refer to _openedFile outside function scope
        global _openedFile

        #Check if a file is opened
        if(_openedFile is None):
            raise Exception("HTML Output: No file is opened to write")

        #Initiate write procedure
        _openedFile.write(line.strip() + "\r\n")

    if(ACTION == "range"):

        rangeHandler.putLine(line)

    if(ACTION == "rangeEnd"):

        rangeHandler.endRangeBlock()



import os

import yaml

import siteData as sd
import dataInterface as di

def parse(configFilePath):

    if(not os.path.isabs(configFilePath)):
        currentDir = os.getcwd()
        configFilePath = os.path.join(currentDir, configFilePath)

    configFile = open(configFilePath, "r")
    configContent = configFile.read()
    parsedContent = yaml.load(configContent)
    _placeData(parsedContent)

def _placeData(data):

    sd.GLOBALS.update(data["globals"])
    sd.DATAFILESOBJECT.update(data["datafiles"])

    sd.setNuoDir(os.path.abspath(data["nuodir"]))

    di.placeData()



import siteData as sd
class Stack:

    def __init__(self):
        self.stack = []

    def push(self, x):
        self.stack.append(x)

    def pop(self):
        return self.stack.pop()

    def top(self):
        return self.stack[len(self.stack) - 1]

    def isEmpty(self):
        return True if (len(self.stack) == 0) else False

#Function to access properties of object
def chainedPropertyAccess(obj, arr = []):

    #Check for unit case
    if(len(arr) == 1):
        try:
            #Unit case, Return Value of chained properties of an object
            return obj[arr[0]]
        except KeyError:
            #Return empty string when interrupt is a KeyError
            return ""
    else:
        #Handle KeyError
        try:
            #Recursive to call access property of property of an object
            return chainedPropertyAccess(obj[arr[0]], arr[1:])
        except KeyError:
            #Return empty string when interrupt is a KeyError
            return ""

def getPostfix(exp):

    postfix = ""
    operators = ["+", "-", "/", "*"]
    pre = {"+": 1, "-": 1, "*": 2, "/": 2}
    stack = Stack()

    for char in exp:
        if(char == '('):
            stack.push(char)
            continue

        if(char not in operators and char is not "(" and char is not ")"):
            postfix += char
            continue

        if(char == ")"):
            while(stack.top() is not "("):
                postfix += " " + stack.pop() + " "
            stack.pop()
            continue

        if(stack.isEmpty() or stack.top() == "(" or pre[stack.top()] < pre[char]):
            stack.push(char)
            continue
        else:
            while(1):
                if(stack.isEmpty() or stack.top() == "(" or pre[stack.top()] < pre[char]):
                    break
                postfix += " " + stack.pop() + " "
            stack.push(char)

    while(not stack.isEmpty()):
        postfix += " " + stack.pop() + " "

    return postfix

def evalPostfix(expression):

    tempExp = expression.split()
    operators = ["+", "-", "/", "*"]
    stack = Stack()

    for op in tempExp:
        if(op in operators):
            x, y = stack.pop(), stack.pop()
            if(type(x) is int or x.isdigit()):
                stack.push(_eval(int(x), int(y), op))
            else:
                x = getValue(x)
                stack.push(_eval(x, y, op))
            continue
        if(op is not " "):
            stack.push(op)

    return stack.top()

def _eval(x, y, op):

    if(op == "+"):
        return y + x

    if(op == "-"):
        return y - x

    if(op == "/"):
        return int(y / x)

    if(op == "*"):
        return y * x

def getValue(ch, rangeData = False, rangeDataObj = {}):

    if(rangeData and ch[0] in rangeDataObj.keys()):
        return chainedPropertyAccess(rangeDataObj, ch)

    if(ch[0] in sd.GLOBALS.keys()):
        return chainedPropertyAccess(sd.GLOBALS, ch)

    if(ch[0] in sd.DATAOBJECT.keys()):
        return chainedPropertyAccess(sd.DATAOBJECT, ch)

    if(ch[0] in sd.DEFINEDOBJECTS.keys()):
        return chainedPropertyAccess(sd.DEFINEDOBJECTS, ch)



from distutils.core import setup
import codecs

import main

setup (

    name = "NuO",
    version = "0.1.3",
    author = "Akshit Grover",
    author_email = "akshit.grover2016@gmail.com",
    description = "Static website templating engine, Generates HTML pages using .nuo template files and data in JSON files.",
    packages = ["."],
    long_description = open("README.rst").read(),
    entry_points={
        "console_scripts": [
            "nuo = main:main"
        ]
    },
    install_requires = [
        "PyYAML"
    ],
    license = "LICENSE",
    url = "https://github.com/akshitgrover/NuO",
    keywords = "static html website parser nuo",
    classifiers = [

        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.6",
    
    ]

)


import os
import json

import siteData as sd

def placeData():

    for key in sd.DATAFILESOBJECT.keys():
        value = sd.DATAFILESOBJECT[key]
        value = os.path.abspath(value)

        dataFile = open(value, "r")
        data = dataFile.read()
        parsedJsonData = json.loads(data)

        sd.DATAOBJECT[key] = parsedJsonData
